# Copyright (C) 2002, 2003 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""Provide an authentication wrapper around archives accessed via
returned results from htdig's htsearch. Access via htdig requires the
user's request present a valid cookie authorizing access to the 
list's archives for private archives. 
This cookie must be obtained by the same process as the user must 
adopt for accessing the archive directly rather than via
htsearch results. Indeed the user should only be able to reach the
search facility, which appears on the list archives front page, if
they have been through the authentication process. However, this code 
prevents someone hand fettling a URL on the browser or using one
given to them by an authorised user,  which might compromise the 
list's privacy.
"""

# this code was derived from the private.py cgi script

import sys
import os
import cgi
import mimetypes
import re

from Mailman import mm_cfg
from Mailman import Utils
from Mailman import MailList
from Mailman import Errors
from Mailman import i18n
from Mailman.htmlformat import *
from Mailman.Logging.Syslog import syslog

# Set up i18n.  Until we know which list is being requested, we use the
# server's default.
_ = i18n._
i18n.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)

errors = {'path': _('The requested document cannot be found.'),
          'data': _('The requested document cannot be read.'),
          'auth': Utils.maketext('htdig_auth_failure.html', dict=None, raw=0)
         }

def true_path(path):
    "Ensure that the path is safe by removing .."
    path = path.replace("../", "")
    path = path.replace("./", "")
    return path[1:]


def make_inserts(listname):
    urlbase =  mm_cfg.DEFAULT_URL or \
               mm_cfg.DEFAULT_URL_PATTERN % mm_cfg.DEFAULT_URL_HOST
    return {
            'mailto': Utils.get_site_email(),
            'listinfo_link': urlbase + '/listinfo/' + listname,
            'referer': os.environ.get('HTTP_REFERER', _('Referer not known')),
            'uri': os.environ.get('REQUEST_URI', _('URI not known')),
           }


def error_quit(listname, error_type):
    d = make_inserts(listname)
    d['error'] = errors[error_type]
    charset = Utils.GetCharSet(mm_cfg.DEFAULT_SERVER_LANGUAGE)
    stuff = 'Content-type: text/html; charset=' + charset + '\n\n'
    stuff += Utils.maketext('htdig_access_error.html', dict=d, 
                         lang=mm_cfg.DEFAULT_SERVER_LANGUAGE)
    print stuff
    sys.exit(0)

    
def main():
    list_info = Utils.GetPathPieces()
    listname = ''
    access_failure = 'path'
    if list_info:
        access_failure = ''
        try:
            path = os.environ.get('PATH_INFO')
            true_filename = os.path.join(mm_cfg.PRIVATE_ARCHIVE_FILE_DIR,
                                         true_path(path))
            # The path should be:
            #  <list name>/<archive name (but not htdig or database)>/<file_name>
            #  <list name>/<file_name which ends in .txt, .txt.gz or .html>
            num_parts = len(list_info)
            if (num_parts == 3) or (num_parts == 2):
                listname = list_info[0].lower()
                try:
                    mlist = MailList.MailList(listname, lock=0)
                except:
                    access_failure = 'list'
                else:
                    if num_parts == 3 and \
                       list_info[1] in ('database', 'htdig'):
                        access_failure = 'path'
                    elif not (os.path.exists(true_filename) and \
                              os.path.isfile(true_filename)):
                        access_failure = 'file'
                    elif num_parts == 2 and \
                         not re.compile(r'\.(html|txt|txt\.gz)$').search(true_filename):
                        access_failure = 'file'
            else:
                access_failure = 'path'
        except:
            access_failure = 'path'

    if access_failure:
        error_quit(listname, 'path')
    
    # We only need to authorize the user if it's a private archive 
    if mlist.archive_private:
        is_auth = mlist.WebAuthenticate((mm_cfg.AuthUser,
                                         mm_cfg.AuthListModerator,
                                         mm_cfg.AuthListAdmin,
                                         mm_cfg.AuthSiteAdmin),
                                        '', '')
        if not is_auth:
            error_quit(listname, 'auth')

    # OK to output the desired file
    try:
        f = open(true_filename, 'r')
    except IOError:
        error_quit(listname, 'data')
    ctype, cencode = mimetypes.guess_type(true_filename)
    if not (ctype or cencode):
        ctype = 'application/octet-stream'
    elif cencode:
        ctype = "application/x-%s" % cencode
    print "Content-type: %s\n\n" % ctype
    while (1):
        data = f.read(16384)
        if data == "": break
        sys.stdout.write(data)
    f.close()
    sys.exit(0)
