# Copyright (C) 2000,2001,2002 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""Outgoing queue runner."""

import time
import errno
import os
from stat import *
from email.Utils import parsedate_tz, mktime_tz, formatdate

from Mailman import mm_cfg
from Mailman import LockFile
from Mailman.Queue.Runner import Runner
from Mailman import MailList
from Mailman import Utils
from Mailman.Archiver import HyperArch

# Part of the Mailman-htdig integration.
# This controls how often _doperiodic() will try to deal with the
# consequences of a remote machine having reindexed mail archives
# and hence the need for the MM machine to update affected lists  
# TOC pages to reflect the datetime when the htdigging was done.
CHECK_REMOTE_RUNDIG_EFFECTS = 10


class ArchRunner(Runner):
    QDIR = mm_cfg.ARCHQUEUE_DIR

    def __init__(self, slice=None, numslices=1):
        Runner.__init__(self, slice, numslices)
        self._periodic_htdig_check = CHECK_REMOTE_RUNDIG_EFFECTS

    def _dispose(self, mlist, msg, msgdata):
        # Support clobber_date, i.e. setting the date in the archive to the
        # received date, not the (potentially bogus) Date: header of the
        # original message.
        clobber = 0
        originaldate = msg.get('date')
        receivedtime = formatdate(msgdata['received_time'])
        if not originaldate:
            clobber = 1
        elif mm_cfg.ARCHIVER_CLOBBER_DATE_POLICY == 1:
            clobber = 1
        elif mm_cfg.ARCHIVER_CLOBBER_DATE_POLICY == 2:
            # what's the timestamp on the original message?
            tup = parsedate_tz(originaldate)
            now = time.time()
            try:
                if not tup:
                    clobber = 1
                elif abs(now - mktime_tz(tup)) > \
                         mm_cfg.ARCHIVER_ALLOWABLE_SANE_DATE_SKEW:
                    clobber = 1
            except ValueError:
                # The likely cause of this is that the year in the Date: field
                # is horribly incorrect, e.g. (from SF bug # 571634):
                # Date: Tue, 18 Jun 0102 05:12:09 +0500
                # Obviously clobber such dates.
                clobber = 1
        if clobber:
            del msg['date']
            del msg['x-original-date']
            msg['Date'] = receivedtime
            if originaldate:
                msg['X-Original-Date'] = originaldate
        # Always put an indication of when we received the message.
        msg['X-List-Received-Date'] = receivedtime
        # Now try to get the list lock
        try:
            mlist.Lock(timeout=mm_cfg.LIST_LOCK_TIMEOUT)
        except LockFile.TimeOutError:
            # oh well, try again later
            return 1
        try:
            mlist.ArchiveMail(msg)
            mlist.Save()
        finally:
            mlist.Unlock()

    def _doperiodic(self):
        """Do some processing `every once in a while'.

        If the mailman-htdig archiving is being used then we want to ensure
        that the TOC page for each list has been updated since the last time
        htdigging of the list was done. This is only necessary if we are
        running htdig on a different machine to Mailman.

        """

        if mm_cfg.USE_HTDIG and mm_cfg.REMOTE_HTDIG:
            self._periodic_htdig_check -= 1
            if self._periodic_htdig_check <= 0: 
                self._periodic_htdig_check = CHECK_REMOTE_RUNDIG_EFFECTS
                listnames = Utils.list_names()
                for name in listnames:
                    mlist = MailList.MailList(name, lock=0)
                    if not mlist.last_post_time > 0: continue
                    arch_dir = mlist.archive_dir()
                    rundig_run_file = os.path.join(arch_dir, 'htdig', 
                                                   'rundig_last_run')
                    toc_file = os.path.join(arch_dir, 'index.html')
                    try:
                        last_rundig_time = os.stat(rundig_run_file)[ST_MTIME]
                        last_toc_time = os.stat(toc_file)[ST_MTIME]
                    except OSError, e:
                        if e.errno <> errno.ENOENT: raise
                    else:
                        if last_rundig_time > last_toc_time:
                            HyperArch.HyperArchive(mlist).write_TOC()

