#! /usr/bin/env perl
# 
# Copyright (C) 2002 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

# this code was derived from the nightly_gzip cron script

my $REMOTE_PRIVATE_ARCHIVE_FILE_DIR = '';
my $HTDIG_RUNDIG_PATH = '';

use strict;
use File::Spec;
use File::stat;
use LWP::Simple;
use Getopt::Long;

my $doc = <<EOT;
Perl version of the script to re-generate the htdig archive search files.

Read INSTALL.htdig-mm to determine if you should be running this script.

This script has to be edited before use to provide values for the
configuration parameters REMOTE_PRIVATE_ARCHIVE_FILE_DIR and
HTDIG_RUNDIG_PATH. The values should be the same as those acquired by
Mailman's python scripts from \$prefix/Mailman/Defaults.py or overridden
in \$prefix/Mailman/mm_cfg.py.

This script should normally be run nightly from cron.  When run from the
command line, the following usage is understood:

Usage: remote_nightly_htdig.pl [-v] [-h] [listnames]

Where:
    --verbose / -v
        print each list as htdig is run for it

    --help / -h
        print this message and exit

    listnames
        Optionally, only runs htdig for the named lists.  Without 
        this, all archivable lists are processed.

EOT

my $VERBOSE = 0;
my $omask;

sub usage {
    my ($code, $msg) = @_;
    print $doc;
    print $msg if $msg;
    umask($omask);
    exit($code);
}

sub main {
    my $help;
    GetOptions("verbose" => \$VERBOSE,
               "help" => \$help);
    if ($help) {
        usage(0);
    }
    # limit to the specified lists?
    my @listnames = ();
    if (scalar(@ARGV)) {
        @listnames = map lc, @ARGV;
    } else {
        opendir(DIR, $REMOTE_PRIVATE_ARCHIVE_FILE_DIR);
        @listnames = grep { $_ ne '.' and $_ ne '..' and ! /\.mbox$/ } readdir DIR;
        closedir(DIR);
    }
    # process all the specified lists
    foreach my $name (@listnames) {
        my $archive_dir = File::Spec->catfile($REMOTE_PRIVATE_ARCHIVE_FILE_DIR, $name);
        next if (-e not $archive_dir);
        # check htdig has been set up for this list and skip it if not
        my $list_htdig_dir = File::Spec->catfile($archive_dir, 'htdig');
        if (not -e $list_htdig_dir) {
            print "Skipping remote htdig for list $name, no htdig setup\n" if $VERBOSE;
            next
        }
        # check if there have been any archive files created since we
        # last ran htdig and skip list if not. well actually we only
        # test if the archive volume directories mod times have changed        
        my $recent_posts = 0;
        my $rundig_run_file = File::Spec->catfile($list_htdig_dir, 'rundig_last_run');
        if (-e $rundig_run_file){      
            my $last_rundig_time = stat($rundig_run_file)->mtime();
            opendir(DIR, $archive_dir);
            my @volumes = grep { $_ ne '.' and $_ ne '..' } readdir DIR;
            closedir(DIR);
            foreach my $volume (@volumes) {
                my $archive_name = File::Spec->catfile($archive_dir, $volume);
                my $last_archive_change = stat($archive_name)->mtime();
                if ($last_archive_change > $last_rundig_time) {
                    $recent_posts = 1;
                    last;
                }
            }
        } else {
            $recent_posts = 1;
        }
        if (not $recent_posts) {
            if ($VERBOSE) {
                print "Skipping htdig for list $name, no recent posts\n";
            }
            next;
        }
        # ok, so running htdig is worthwhile
        if ($VERBOSE) {
            print "htdig'ing archive of list $name\n";
        }
        my $htdig_conf_file = File::Spec->catfile($list_htdig_dir, $name.'.conf');
        my @cmd = ($HTDIG_RUNDIG_PATH, '-c', $htdig_conf_file);
        my $status = system(@cmd) >> 8 & 0xFF;
        if ($status) {
            print "rundig failed for list, $name, exit code, $status\n";
        } else {
            system(("touch", $rundig_run_file));
        }
    }
}

if (-x $HTDIG_RUNDIG_PATH  and
    -d $REMOTE_PRIVATE_ARCHIVE_FILE_DIR) {
    # we're only going to run the nightly rundig if we have a sensible
    # set of configuration variables and we know where rundig is.
    $omask = umask;
    umask(002);
    eval { main() };
    my $res = $@;
    umask($omask);
    die $res if ($res);
} else {
    die "Invalid configuration variables.\nEdit this script in accordance with INSTALL.htdig-mm\n";
}
